import events = require('@aws-cdk/aws-events');
import iam = require('@aws-cdk/aws-iam');
import cdk = require('@aws-cdk/cdk');
import { Artifact } from './artifact';
export declare enum ActionCategory {
    Source = "Source",
    Build = "Build",
    Test = "Test",
    Approval = "Approval",
    Deploy = "Deploy",
    Invoke = "Invoke"
}
/**
 * Specifies the constraints on the number of input and output
 * artifacts an action can have.
 *
 * The constraints for each action type are documented on the
 * {@link https://docs.aws.amazon.com/codepipeline/latest/userguide/reference-pipeline-structure.html Pipeline Structure Reference} page.
 */
export interface ActionArtifactBounds {
    readonly minInputs: number;
    readonly maxInputs: number;
    readonly minOutputs: number;
    readonly maxOutputs: number;
}
export declare function defaultBounds(): ActionArtifactBounds;
/**
 * The API of Stage used internally by the CodePipeline Construct.
 * You should never need to call any of the methods inside of it yourself.
 */
export interface IInternalStage {
    /**
     * Adds an Action to this Stage.
     *
     * @param action the Action to add to this Stage
     */
    _attachAction(action: Action): void;
    /**
     * Generates a unique output artifact name for the given Action.
     *
     * @param action the Action to generate the output artifact name for
     */
    _generateOutputArtifactName(action: Action): string;
    /**
     * Finds an input artifact for the given Action.
     * The chosen artifact will be the output artifact of the
     * last Action in the Pipeline
     * (up to the Stage this Action belongs to)
     * with the highest runOrder that has an output artifact.
     *
     * @param action the Action to find the input artifact for
     */
    _findInputArtifact(action: Action): Artifact;
}
/**
 * The abstract view of an AWS CodePipeline as required and used by Actions.
 * It extends {@link events.IEventRuleTarget},
 * so this interface can be used as a Target for CloudWatch Events.
 */
export interface IPipeline extends cdk.IConstruct, events.IEventRuleTarget {
    /**
     * The name of the Pipeline.
     */
    readonly pipelineName: string;
    /**
     * The ARN of the Pipeline.
     */
    readonly pipelineArn: string;
    /**
     * The service Role of the Pipeline.
     */
    readonly role: iam.Role;
    /**
     * Grants read permissions to the Pipeline's S3 Bucket to the given Identity.
     *
     * @param identity the IAM Identity to grant the permissions to
     */
    grantBucketRead(identity?: iam.IPrincipal): void;
    /**
     * Grants read & write permissions to the Pipeline's S3 Bucket to the given Identity.
     *
     * @param identity the IAM Identity to grant the permissions to
     */
    grantBucketReadWrite(identity?: iam.IPrincipal): void;
}
/**
 * The abstract interface of a Pipeline Stage that is used by Actions.
 */
export interface IStage extends cdk.IConstruct {
    /**
     * The physical, human-readable name of this Pipeline Stage.
     */
    readonly name: string;
    /**
     * The Pipeline this Stage belongs to.
     */
    readonly pipeline: IPipeline;
    /**
     * The API of Stage used internally by the CodePipeline Construct.
     * You should never need to call any of the methods inside of it yourself.
     */
    readonly _internal: IInternalStage;
}
/**
 * Common properties shared by all Actions.
 */
export interface CommonActionProps {
    /**
     * The runOrder property for this Action.
     * RunOrder determines the relative order in which multiple Actions in the same Stage execute.
     *
     * @default 1
     * @see https://docs.aws.amazon.com/codepipeline/latest/userguide/reference-pipeline-structure.html
     */
    runOrder?: number;
}
/**
 * Common properties shared by all Action Constructs.
 */
export interface CommonActionConstructProps {
    /**
     * The Pipeline Stage to add this Action to.
     */
    stage: IStage;
}
/**
 * Construction properties of the low-level {@link Action Action class}.
 */
export interface ActionProps extends CommonActionProps, CommonActionConstructProps {
    category: ActionCategory;
    provider: string;
    /**
     * The region this Action resides in.
     *
     * @default the Action resides in the same region as the Pipeline
     */
    region?: string;
    /**
     * The service role that is assumed during execution of action.
     * This role is not mandatory, however more advanced configuration
     * may require specifying it.
     *
     * @see https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-codepipeline-pipeline-stages-actions.html
     */
    role?: iam.IRole;
    artifactBounds: ActionArtifactBounds;
    configuration?: any;
    version?: string;
    owner?: string;
}
/**
 * Low-level class for generic CodePipeline Actions.
 * It is recommended that concrete types are used instead, such as {@link codecommit.PipelineSourceAction} or
 * {@link codebuild.PipelineBuildAction}.
 */
export declare abstract class Action extends cdk.Construct {
    /**
     * The category of the action.
     * The category defines which action type the owner
     * (the entity that performs the action) performs.
     */
    readonly category: ActionCategory;
    /**
     * The service provider that the action calls.
     */
    readonly provider: string;
    /**
     * The AWS region the given Action resides in.
     * Note that a cross-region Pipeline requires replication buckets to function correctly.
     * You can provide their names with the {@link PipelineProps#crossRegionReplicationBuckets} property.
     * If you don't, the CodePipeline Construct will create new Stacks in your CDK app containing those buckets,
     * that you will need to `cdk deploy` before deploying the main, Pipeline-containing Stack.
     *
     * @default the Action resides in the same region as the Pipeline
     */
    readonly region?: string;
    /**
     * The action's configuration. These are key-value pairs that specify input values for an action.
     * For more information, see the AWS CodePipeline User Guide.
     *
     * http://docs.aws.amazon.com/codepipeline/latest/userguide/reference-pipeline-structure.html#action-requirements
     */
    readonly configuration?: any;
    /**
     * The service role that is assumed during execution of action.
     * This role is not mandatory, however more advanced configuration
     * may require specifying it.
     *
     * @see https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-codepipeline-pipeline-stages-actions.html
     */
    readonly role?: iam.IRole;
    /**
     * The order in which AWS CodePipeline runs this action.
     * For more information, see the AWS CodePipeline User Guide.
     *
     * https://docs.aws.amazon.com/codepipeline/latest/userguide/reference-pipeline-structure.html#action-requirements
     */
    readonly runOrder: number;
    readonly owner: string;
    readonly version: string;
    private readonly _actionInputArtifacts;
    private readonly _actionOutputArtifacts;
    private readonly artifactBounds;
    private readonly stage;
    constructor(scope: cdk.Construct, id: string, props: ActionProps);
    onStateChange(name: string, target?: events.IEventRuleTarget, options?: events.EventRuleProps): events.EventRule;
    readonly _inputArtifacts: Artifact[];
    readonly _outputArtifacts: Artifact[];
    protected validate(): string[];
    protected addOutputArtifact(name?: string): Artifact;
    protected addInputArtifact(artifact?: Artifact): Action;
}
